# Copyright (C) 2012-2024, Stefan Kanthak <stefan.kanthak@nexgo.de>

# * The software is provided "as is" without any warranty, neither express
#   nor implied.
# * In no event will the author be held liable for any damage(s) arising
#   from the use of the software.
# * Redistribution of the software is allowed only in unmodified form.
# * Permission is granted to use the software solely for personal private
#   and non-commercial purposes.
# * An individuals use of the software in his or her capacity or function
#   as an agent, (independent) contractor, employee, member or officer of
#   a business, corporation or organization (commercial or non-commercial)
#   does not qualify as personal private and non-commercial purpose.
# * Without written approval from the author the software must not be used
#   for a business, for commercial, corporate, governmental, military or
#   organizational purposes of any kind, or in a commercial, corporate,
#   governmental, military or organizational environment of any kind.

.SUFFIXES:

gui: WPBT.EXE
	$(MAKEDIR)\WPBT.EXE

cli: WPBT.COM
	$(MAKEDIR)\WPBT.COM

clobber: clean
	Erase WPBT.COM WPBT.EXE

clean:
	Erase WPBT.DOS WPBT.OBJ WPBT.TMP

WPBT.COM: WPBT.TMP
	LINK.EXE /LINK /BREPRO /DEFAULTLIB:KERNEL32.LIB /ENTRY:mainCRTStartup /FIXED /LARGEADDRESSAWARE /MERGE:.rdata=.const /MERGE:.text=.code /NOCOFFGRPINFO /NOLOGO /NXCOMPAT /OPT:REF /OSVERSION:5.2 /OUT:$@ /RELEASE /STUB:WPBT.DOS /SUBSYSTEM:CONSOLE /SWAPRUN:CD,NET /TEST /VERSION:0.815 $**

WPBT.EXE: WPBT.OBJ
#	LINK.EXE /LINK /BREPRO /DEFAULTLIB:KERNEL32.LIB /DEFAULTLIB:USER32.LIB /ENTRY:WinMainCRTStartup /FIXED /LARGEADDRESSAWARE /MERGE:.rdata=.const /MERGE:.text=.code /NOCOFFGRPINFO /NOLOGO /NXCOMPAT /OPT:REF /OSVERSION:5.2 /OUT:$@ /RELEASE /STACK:1048576,65536 /STUB:WPBT.DOS /SUBSYSTEM:WINDOWS /SWAPRUN:CD,NET /TEST /VERSION:0.815 $**
	LINK.EXE /LINK /BREPRO /DEFAULTLIB:KERNEL32.LIB /DEFAULTLIB:USER32.LIB /DYNAMICBASE /ENTRY:WinMainCRTStartup /LARGEADDRESSAWARE /MERGE:.rdata=.const /MERGE:.text=.code /NOCOFFGRPINFO /NOLOGO /NXCOMPAT /OPT:REF /OSVERSION:5.2 /OUT:$@ /RELEASE /STACK:1048576,65536 /STUB:WPBT.DOS /SUBSYSTEM:WINDOWS /SWAPRUN:CD,NET /TEST /VERSION:0.815 $**

WPBT.DOS: WPBT.MAK
	CERTUTIL.EXE /DecodeHex /F /V << $@
4D 5A 90 00 01 00 00 00 04 00 00 00 FF FF 00 00   MZ..............
E0 00 00 00 43 00 00 00 40 00 00 00 00 00 00 00   ....C...@.......
00 00 00 00 19 57 04 27 00 00 00 00 00 00 00 00   .....W.'........
00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00   ................
28 43 29 6F 70 79 72 69 67 68 74 20 32 30 30 34   (C)opyright 2004
2D 32 30 32 34 2C 20 53 74 65 66 61 6E 20 4B 61   -2024, Stefan Ka
6E 74 68 61 6B 20 3C 73 74 65 66 61 6E 2E 6B 61   nthak <stefan.ka
6E 74 68 61 6B 40 6E 65 78 67 6F 2E 64 65 3E 0D   nthak@nexgo.de>.
0A 07 24 0E 1F 33 D2 B4 09 CD 21 B8 01 4C CD 21   .....3....!..L.!
<<NOKEEP

WPBT.OBJ: WPBT.MAK
	CL.EXE /Brepro /c /Fo$@ /GA /GF /Gs69632 /Gw /Gy /nologo /Ox /Oy /Tc<< /W4 /we4013 /Zl
// Copyright (C) 2012-2024, Stefan Kanthak <stefan.kanthak@nexgo.de>

// https://msdn.microsoft.com/en-us/library/7f0aews7.aspx

#pragma comment(compiler)

#define STRICT
#define UNICODE
#define WIN32_LEAN_AND_MEAN

#include <windows.h>

typedef	struct	_wpbt		// Windows Platform Binary Table
{
	DWORD	Signature;	// "WPBT" = 0x54445057 = 'TBPW'
	DWORD	Length;		// size of entire WPBT
	BYTE	Revision;
	BYTE	Checksum;	// checksum of entire WPBT
	CHAR	OEMId[6];	// OEM identification
	CHAR	OEMTableId[8];
	DWORD	OEMRevision;
	CHAR	CreatorId[4];	// creator identification
	DWORD	CreatorRevision;

	DWORD	HandoffMemorySize;
	DWORD64	HandoffMemoryLocation;
	BYTE	ContentLayout;
	BYTE	ContentType;
	WORD	CommandLineLength;
	WCHAR	CommandLine[32768];
} WPBT;

// https://msdn.microsoft.com/en-us/library/f9t8842e.aspx

__declspec(noreturn)
__declspec(safebuffers)
VOID	CDECL	WinMainCRTStartup(VOID)
{
	WPBT	wpbt;

	// https://msdn.microsoft.com/en-us/library/ms724379.aspx

	UINT	size = GetSystemFirmwareTable('ACPI', 'TBPW', &wpbt, sizeof(wpbt));

	DWORD	length;
	BYTE	checksum;

	if (size == 0U)
		MessageBoxExW(HWND_DESKTOP,
		              L"No \'WPBT\' found!",
		              L"\'WPBT\' Command Line Reader",
		              MB_OK | MB_ICONQUESTION,
		              MAKELANGID(LANG_ENGLISH, SUBLANG_NEUTRAL));
	else if (size > sizeof(wpbt))
		MessageBoxExW(HWND_DESKTOP,
		              L"Error reading \'WPBT\'!",
		              L"\'WPBT\' Command Line Reader",
		              MB_OK | MB_ICONERROR,
		              MAKELANGID(LANG_ENGLISH, SUBLANG_NEUTRAL));
	else
	{
		for (length = wpbt.Length, checksum = 0;
		     length > 0UL;
		     checksum += ((BYTE *) &wpbt)[--length])
			continue;

		if ((checksum != 0)
		 || (wpbt.Signature != 'TBPW')
		 || (wpbt.Length < sizeof(wpbt) - sizeof(wpbt.CommandLine))
		 || (wpbt.ContentLayout != 0x01)
		 || (wpbt.ContentType != 0x01))
			MessageBoxExW(HWND_DESKTOP,
			              L"Invalid or unknown \'WPBT\' found!",
			              L"\'WPBT\' Command Line Reader",
			              MB_OK | MB_ICONWARNING,
			              MAKELANGID(LANG_ENGLISH, SUBLANG_NEUTRAL));
		else
		{
			wpbt.CommandLine[wpbt.CommandLineLength / sizeof(*wpbt.CommandLine)] = L'\0';

			// https://msdn.microsoft.com/en-us/library/ms645507.aspx

			MessageBoxExW(HWND_DESKTOP,
			              wpbt.CommandLine,
			              L"\'WPBT\' Command Line Reader",
			              MB_OK | MB_ICONINFORMATION,
			              MAKELANGID(LANG_ENGLISH, SUBLANG_NEUTRAL));
		}
	}

	// https://msdn.microsoft.com/en-us/library/ms682658.aspx
	// https://msdn.microsoft.com/en-us/library/ms679360.aspx

	ExitProcess(GetLastError());
}
<<NOKEEP

WPBT.TMP: WPBT.MAK
	CL.EXE /Brepro /c /Fo$@ /GA /GF /Gw /Gy /nologo /Ox /Oy /Tc<< /W4 /we4013 /Zl
// Copyright (C) 2012-2024, Stefan Kanthak <stefan.kanthak@nexgo.de>

// https://msdn.microsoft.com/en-us/library/7f0aews7.aspx

#pragma comment(compiler)

#define STRICT
#define WIN32_LEAN_AND_MEAN

#include <windows.h>

// https://msdn.microsoft.com/en-us/library/f9t8842e.aspx

__declspec(noreturn)
VOID	CDECL	mainCRTStartup(VOID)
{
	// https://msdn.microsoft.com/en-us/library/ms682658.aspx
	// https://msdn.microsoft.com/en-us/library/ms724379.aspx
	// https://msdn.microsoft.com/en-us/library/ms679360.aspx

	ExitProcess(GetSystemFirmwareTable('ACPI', 'TBPW', NULL, 0UL) == 0U ? GetLastError() : 0U);
}
<<NOKEEP
